searchData={"items":[{"type":"module","title":"Elixact","doc":"Elixact is a schema definition and validation library for Elixir.\n\nIt provides a DSL for defining schemas with rich metadata, validation rules,\nand JSON Schema generation capabilities.","ref":"Elixact.html"},{"type":"module","title":"Example - Elixact","doc":"defmodule UserSchema do\n      use Elixact\n\n      schema \"User account information\" do\n        field :name, :string do\n          description \"User's full name\"\n          example \"John Doe\"\n          required true\n          min_length 2\n        end\n\n        field :age, :integer do\n          description \"User's age in years\"\n          optional true\n          gt 0\n          lt 150\n        end\n\n        field :email, Types.Email do\n          description \"User's email address\"\n          required true\n        end\n\n        config do\n          title \"User Schema\"\n          strict true\n        end\n      end\n    end\n\nThe schema can then be used for validation and JSON Schema generation:\n\n    # Validation\n    {:ok, user} = UserSchema.validate(%{\n      name: \"John Doe\",\n      email: \"john@example.com\",\n      age: 30\n    })\n\n    # JSON Schema generation\n    json_schema = UserSchema.json_schema()","ref":"Elixact.html#module-example"},{"type":"macro","title":"Elixact.__using__/1","doc":"Configures a module to be a Elixact schema.\n\nThis macro sets up the necessary module attributes and imports\nthe schema DSL functions.","ref":"Elixact.html#__using__/1"},{"type":"module","title":"Elixact.Error","doc":"Structured error representation for Elixact validation errors.","ref":"Elixact.Error.html"},{"type":"function","title":"Elixact.Error.format/1","doc":"Formats an error into a human-readable string.","ref":"Elixact.Error.html#format/1"},{"type":"function","title":"Example - Elixact.Error.format/1","doc":"iex> Error.format(%Error{path: [:user, :email], code: :format, message: \"invalid format\"})\n    \"user.email: invalid format\"","ref":"Elixact.Error.html#format/1-example"},{"type":"function","title":"Elixact.Error.new/3","doc":"Creates a new validation error.","ref":"Elixact.Error.html#new/3"},{"type":"function","title":"Parameters - Elixact.Error.new/3","doc":"* `path` - Path to the field that caused the error\n  * `code` - Error code identifying the type of error\n  * `message` - Human-readable error message","ref":"Elixact.Error.html#new/3-parameters"},{"type":"function","title":"Example - Elixact.Error.new/3","doc":"iex> Error.new([:user, :email], :format, \"invalid email format\")\n    %Error{path: [:user, :email], code: :format, message: \"invalid email format\"}","ref":"Elixact.Error.html#new/3-example"},{"type":"type","title":"Elixact.Error.t/0","doc":"","ref":"Elixact.Error.html#t:t/0"},{"type":"module","title":"Elixact.FieldMeta","doc":"Struct for field metadata","ref":"Elixact.FieldMeta.html"},{"type":"module","title":"Elixact.JsonSchema","doc":"","ref":"Elixact.JsonSchema.html"},{"type":"function","title":"Elixact.JsonSchema.from_schema/1","doc":"","ref":"Elixact.JsonSchema.html#from_schema/1"},{"type":"module","title":"Elixact.JsonSchema.ReferenceStore","doc":"Manages schema references and definitions for JSON Schema generation.","ref":"Elixact.JsonSchema.ReferenceStore.html"},{"type":"function","title":"Elixact.JsonSchema.ReferenceStore.add_definition/3","doc":"","ref":"Elixact.JsonSchema.ReferenceStore.html#add_definition/3"},{"type":"function","title":"Elixact.JsonSchema.ReferenceStore.add_reference/2","doc":"","ref":"Elixact.JsonSchema.ReferenceStore.html#add_reference/2"},{"type":"function","title":"Elixact.JsonSchema.ReferenceStore.get_definitions/1","doc":"","ref":"Elixact.JsonSchema.ReferenceStore.html#get_definitions/1"},{"type":"function","title":"Elixact.JsonSchema.ReferenceStore.get_references/1","doc":"","ref":"Elixact.JsonSchema.ReferenceStore.html#get_references/1"},{"type":"function","title":"Elixact.JsonSchema.ReferenceStore.has_definition?/2","doc":"","ref":"Elixact.JsonSchema.ReferenceStore.html#has_definition?/2"},{"type":"function","title":"Elixact.JsonSchema.ReferenceStore.has_reference?/2","doc":"","ref":"Elixact.JsonSchema.ReferenceStore.html#has_reference?/2"},{"type":"function","title":"Elixact.JsonSchema.ReferenceStore.ref_path/1","doc":"","ref":"Elixact.JsonSchema.ReferenceStore.html#ref_path/1"},{"type":"function","title":"Elixact.JsonSchema.ReferenceStore.start_link/0","doc":"","ref":"Elixact.JsonSchema.ReferenceStore.html#start_link/0"},{"type":"function","title":"Elixact.JsonSchema.ReferenceStore.stop/1","doc":"","ref":"Elixact.JsonSchema.ReferenceStore.html#stop/1"},{"type":"module","title":"Elixact.JsonSchema.TypeMapper","doc":"Converts Elixact type definitions to JSON Schema type definitions.","ref":"Elixact.JsonSchema.TypeMapper.html"},{"type":"function","title":"Elixact.JsonSchema.TypeMapper.to_json_schema/2","doc":"","ref":"Elixact.JsonSchema.TypeMapper.html#to_json_schema/2"},{"type":"module","title":"Elixact.Schema","doc":"Schema DSL for defining data schemas with validation rules and metadata.","ref":"Elixact.Schema.html"},{"type":"macro","title":"Elixact.Schema.choices/1","doc":"","ref":"Elixact.Schema.html#choices/1"},{"type":"macro","title":"Elixact.Schema.config/1","doc":"","ref":"Elixact.Schema.html#config/1"},{"type":"macro","title":"Elixact.Schema.config_description/1","doc":"","ref":"Elixact.Schema.html#config_description/1"},{"type":"macro","title":"Elixact.Schema.default/1","doc":"","ref":"Elixact.Schema.html#default/1"},{"type":"macro","title":"Elixact.Schema.description/1","doc":"","ref":"Elixact.Schema.html#description/1"},{"type":"macro","title":"Elixact.Schema.example/1","doc":"","ref":"Elixact.Schema.html#example/1"},{"type":"macro","title":"Elixact.Schema.examples/1","doc":"","ref":"Elixact.Schema.html#examples/1"},{"type":"macro","title":"Elixact.Schema.field/3","doc":"","ref":"Elixact.Schema.html#field/3"},{"type":"macro","title":"Elixact.Schema.format/1","doc":"","ref":"Elixact.Schema.html#format/1"},{"type":"macro","title":"Elixact.Schema.gt/1","doc":"","ref":"Elixact.Schema.html#gt/1"},{"type":"macro","title":"Elixact.Schema.gteq/1","doc":"","ref":"Elixact.Schema.html#gteq/1"},{"type":"macro","title":"Elixact.Schema.lt/1","doc":"","ref":"Elixact.Schema.html#lt/1"},{"type":"macro","title":"Elixact.Schema.lteq/1","doc":"","ref":"Elixact.Schema.html#lteq/1"},{"type":"macro","title":"Elixact.Schema.max_items/1","doc":"","ref":"Elixact.Schema.html#max_items/1"},{"type":"macro","title":"Elixact.Schema.max_length/1","doc":"","ref":"Elixact.Schema.html#max_length/1"},{"type":"macro","title":"Elixact.Schema.min_items/1","doc":"","ref":"Elixact.Schema.html#min_items/1"},{"type":"macro","title":"Elixact.Schema.min_length/1","doc":"","ref":"Elixact.Schema.html#min_length/1"},{"type":"macro","title":"Elixact.Schema.optional/0","doc":"","ref":"Elixact.Schema.html#optional/0"},{"type":"macro","title":"Elixact.Schema.required/0","doc":"","ref":"Elixact.Schema.html#required/0"},{"type":"macro","title":"Elixact.Schema.schema/2","doc":"","ref":"Elixact.Schema.html#schema/2"},{"type":"macro","title":"Elixact.Schema.strict/1","doc":"","ref":"Elixact.Schema.html#strict/1"},{"type":"macro","title":"Elixact.Schema.title/1","doc":"","ref":"Elixact.Schema.html#title/1"},{"type":"behaviour","title":"Elixact.Type","doc":"Behaviour and macros for defining custom types.","ref":"Elixact.Type.html"},{"type":"callback","title":"Elixact.Type.json_schema/0","doc":"","ref":"Elixact.Type.html#c:json_schema/0"},{"type":"callback","title":"Elixact.Type.type_definition/0","doc":"","ref":"Elixact.Type.html#c:type_definition/0"},{"type":"callback","title":"Elixact.Type.validate/1","doc":"","ref":"Elixact.Type.html#c:validate/1"},{"type":"module","title":"Elixact.Types","doc":"Core type system for Elixact schemas.\n\nProvides functions for defining and working with types:\n- Basic types (:string, :integer, :float, :boolean)\n- Complex types (arrays, maps, unions)\n- Type constraints\n- Type validation\n- Type coercion","ref":"Elixact.Types.html"},{"type":"module","title":"Basic Types - Elixact.Types","doc":"# String type\n    Types.string()\n\n    # Integer type with constraints\n    Types.integer()\n    |> Types.with_constraints(gt: 0, lt: 100)","ref":"Elixact.Types.html#module-basic-types"},{"type":"module","title":"Complex Types - Elixact.Types","doc":"# Array of strings\n    Types.array(Types.string())\n\n    # Map with string keys and integer values\n    Types.map(Types.string(), Types.integer())\n\n    # Union of types\n    Types.union([Types.string(), Types.integer()])","ref":"Elixact.Types.html#module-complex-types"},{"type":"module","title":"Type Constraints - Elixact.Types","doc":"Constraints can be added to types to enforce additional rules:\n\n    Types.string()\n    |> Types.with_constraints([\n      min_length: 3,\n      max_length: 10,\n      format: ~r/^[a-z]+$/\n    ])","ref":"Elixact.Types.html#module-type-constraints"},{"type":"function","title":"Elixact.Types.array/1","doc":"","ref":"Elixact.Types.html#array/1"},{"type":"function","title":"Elixact.Types.boolean/0","doc":"","ref":"Elixact.Types.html#boolean/0"},{"type":"function","title":"Elixact.Types.coerce/2","doc":"","ref":"Elixact.Types.html#coerce/2"},{"type":"function","title":"Elixact.Types.float/0","doc":"","ref":"Elixact.Types.html#float/0"},{"type":"function","title":"Elixact.Types.integer/0","doc":"","ref":"Elixact.Types.html#integer/0"},{"type":"function","title":"Elixact.Types.map/2","doc":"","ref":"Elixact.Types.html#map/2"},{"type":"function","title":"Elixact.Types.normalize_type/1","doc":"","ref":"Elixact.Types.html#normalize_type/1"},{"type":"function","title":"Elixact.Types.ref/1","doc":"","ref":"Elixact.Types.html#ref/1"},{"type":"function","title":"Elixact.Types.string/0","doc":"","ref":"Elixact.Types.html#string/0"},{"type":"function","title":"Elixact.Types.type/1","doc":"","ref":"Elixact.Types.html#type/1"},{"type":"function","title":"Elixact.Types.union/1","doc":"","ref":"Elixact.Types.html#union/1"},{"type":"function","title":"Elixact.Types.validate/2","doc":"","ref":"Elixact.Types.html#validate/2"},{"type":"function","title":"Elixact.Types.with_constraints/2","doc":"","ref":"Elixact.Types.html#with_constraints/2"},{"type":"type","title":"Elixact.Types.type_definition/0","doc":"","ref":"Elixact.Types.html#t:type_definition/0"},{"type":"module","title":"Elixact.Validator","doc":"Validates values against type definitions and schemas.","ref":"Elixact.Validator.html"},{"type":"function","title":"Elixact.Validator.validate/3","doc":"Validates a value against a type definition.","ref":"Elixact.Validator.html#validate/3"},{"type":"function","title":"Elixact.Validator.validate_schema/3","doc":"Validates data against a schema module, checking for required fields,\nfield-level validations, and strict mode constraints if enabled.\n\nReturns {:ok, validated_data} on success or {:error, error} on validation failures.","ref":"Elixact.Validator.html#validate_schema/3"},{"type":"function","title":"Parameters - Elixact.Validator.validate_schema/3","doc":"- schema: Schema module to validate against\n  - data: Data to validate\n  - path: Current validation path for error messages. Defaults to `[]`.","ref":"Elixact.Validator.html#validate_schema/3-parameters"},{"type":"exception","title":"Elixact.ValidationError","doc":"Exception raised when schema validation fails.","ref":"Elixact.ValidationError.html"},{"type":"extras","title":"Elixact","doc":"# Elixact\n\nElixact is a powerful schema definition and validation library for Elixir, inspired by Python's Pydantic.\nIt provides a rich DSL for defining schemas with strong type validation, automatic JSON Schema generation, and excellent developer experience.","ref":"readme.html"},{"type":"extras","title":"Features - Elixact","doc":"- 🎯 **Rich Schema DSL** - Intuitive and expressive schema definitions\n- 🔍 **Strong Type Validation** - Comprehensive validation for basic and complex types\n- 📊 **JSON Schema Support** - Automatic generation of JSON Schema from your Elixir schemas\n- 🧩 **Custom Types** - Easily define reusable custom types\n- 🎄 **Nested Schemas** - Support for deeply nested data structures\n- ⛓️ **Field Constraints** - Rich set of built-in constraints\n- 🚨 **Structured Errors** - Clear and actionable error messages","ref":"readme.html#features"},{"type":"extras","title":"Installation - Elixact","doc":"Add `elixact` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:elixact, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start - Elixact","doc":"","ref":"readme.html#quick-start"},{"type":"extras","title":"Basic Schema Definition - Elixact","doc":"```elixir\ndefmodule UserSchema do\n  use Elixact\n\n  schema \"User account information\" do\n    field :name, :string do\n      description \"User's full name\"\n      min_length 2\n      max_length 50\n    end\n\n    field :age, :integer do\n      description \"User's age\"\n      gt 0\n      lt 150\n      optional true\n    end\n\n    field :email, :string do\n      format ~r/^[^\\s]+@[^\\s]+$/\n    end\n\n    field :tags, {:array, :string} do\n      description \"User tags\"\n      min_items 0\n      max_items 5\n      default []\n    end\n\n    config do\n      title \"User Schema\"\n      strict true\n    end\n  end\nend\n```","ref":"readme.html#basic-schema-definition"},{"type":"extras","title":"Data Validation - Elixact","doc":"```elixir\n# Validate data\ncase UserSchema.validate(%{\n  name: \"John Doe\",\n  email: \"john@example.com\",\n  age: 30,\n  tags: [\"admin\"]\n}) do\n  {:ok, validated_data} ->\n    # Use validated data\n    IO.inspect(validated_data)\n\n  {:error, errors} ->\n    # Handle validation errors\n    Enum.each(errors, &IO.puts(Elixact.Error.format(&1)))\nend\n\n# Or use bang version which raises on error\nvalidated = UserSchema.validate!(data)\n```","ref":"readme.html#data-validation"},{"type":"extras","title":"Complex Types - Elixact","doc":"```elixir\ndefmodule ComplexSchema do\n  use Elixact\n\n  schema do\n    # Array of maps\n    field :metadata, {:array, {:map, {:string, :any}}} do\n      min_items 1\n      description \"Metadata entries\"\n    end\n\n    # Union type\n    field :id, {:union, [:string, :integer]} do\n      description \"User ID (string or integer)\"\n    end\n\n    # Nested schema\n    field :address, AddressSchema do\n      optional true\n    end\n\n    # Map with specific key/value types\n    field :settings, {:map, {:string, {:union, [:string, :boolean, :integer]}}} do\n      description \"User settings\"\n      default %{}\n    end\n  end\nend\n```","ref":"readme.html#complex-types"},{"type":"extras","title":"Custom Types - Elixact","doc":"```elixir\ndefmodule Types.Email do\n  use Elixact.Type\n\n  def type_definition do\n    Elixact.Types.string()\n    |> Elixact.Types.with_constraints([\n      format: ~r/^[^\\s]+@[^\\s]+$/\n    ])\n  end\n\n  def json_schema do\n    %{\n      \"type\" => \"string\",\n      \"format\" => \"email\",\n      \"pattern\" => \"^[^\\\\s]+@[^\\\\s]+$\"\n    }\n  end\nend\n```","ref":"readme.html#custom-types"},{"type":"extras","title":"JSON Schema Generation - Elixact","doc":"```elixir\n# Generate JSON Schema\njson_schema = Elixact.JsonSchema.from_schema(UserSchema)\n\n# The schema can be used with any JSON Schema validator\njson = Jason.encode!(json_schema)\n```","ref":"readme.html#json-schema-generation"},{"type":"extras","title":"Available Types - Elixact","doc":"- Basic Types: `:string`, `:integer`, `:float`, `:boolean`, `:any`\n- Complex Types:\n  - Arrays: `{:array, type}`\n  - Maps: `{:map, {key_type, value_type}}`\n  - Unions: `{:union, [type1, type2, ...]}`\n  - Custom Types: Any module implementing `Elixact.Type` behaviour\n  - Nested Schemas: References to other schema modules","ref":"readme.html#available-types"},{"type":"extras","title":"Field Constraints - Elixact","doc":"- Strings: `min_length`, `max_length`, `format` (regex)\n- Numbers: `gt`, `lt`, `gteq`, `lteq`\n- Arrays: `min_items`, `max_items`\n- General: `required`, `optional`, `default`, `choices`","ref":"readme.html#field-constraints"},{"type":"extras","title":"Error Handling - Elixact","doc":"Elixact provides structured error messages with path information:\n\n```elixir\n{:error, [\n  %Elixact.Error{\n    path: [:email],\n    code: :format,\n    message: \"invalid email format\"\n  }\n]}\n```","ref":"readme.html#error-handling"},{"type":"extras","title":"Contributing - Elixact","doc":"1. Fork it\n2. Create your feature branch (`git checkout -b feature/my-feature`)\n3. Commit your changes (`git commit -am 'Add some feature'`)\n4. Push to the branch (`git push origin feature/my-feature`)\n5. Create a new Pull Request","ref":"readme.html#contributing"},{"type":"extras","title":"License - Elixact","doc":"This project is licensed under the MIT License - see the [LICENSE](LICENSE) file for details.","ref":"readme.html#license"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}